clear
tempfile tmp
set obs 2000
gen byte period = 0 //pre-treatment
gen long id = _n
gen byte gender = uniform() > 0.5
gen age = uniform()
gen fitness = normal(gender*0.25 - age + invnorm(uniform())*0.1) // unobserved selection
gen weight = normal(-gender*0.25 + age*0.25 - fitness*0.25 + invnorm(uniform())*0.1)
gen treated = normal(fitness + invnorm(uniform())*0.25) > 0.73
save `tmp'
replace period = 1 // after treatment
replace weight = weight + weight*(uniform()-0.5)*0.2 - weight*(fitness-0.5)*0.25
append using `tmp'
sort id period
replace weight = int(30.5+100*weight)
replace age = int(18.5+50*age)
sum age
gen agegroup = autocode(age,5,r(min),r(max))
sum weight
gen weightgroup = autocode(weight,5,r(min),r(max)) 
egen long coarsecell = group(agegroup gender weightgroup)
gen effect = treated*period // treatment effect (interaction term for DiD)
probit treated age gender weight if period == 0 // omitting "unobserved" selection 
predict score // propensity score

// Copying and Non-Copying Score-based Neighborhood Matching
// comparing sum of weights
ultimatch score if period == 0, tr(treated) report(score age weight gender fitness) unmatched copy
sum _weight if treated == 0
di r(sum) // sum of weights equals number of matched treated observations
drop if _copy == 1
drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness)
sum _weight if treated == 0 // compare the weights
di r(sum) 

// Single Score-based Neighborhood Matching
// with single draw
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) single support

// Score-based Percentile Rank Neighborhood Matching
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) rank

// Supported Score-based Neighborhood Matching
// with exact matching of gender
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) exact(gender) support

// Supported Score-based Neighborhood Matching
// controlling for gender with an expression (same as above but less efficient)
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) exp(gender == t.gender) support

// Supported Score-based Neighborhood Matching
// with percentile rank limitation and common support
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) limit(age weight) support

// Score-based Neighborhood Matching (multiple counterfactuals)
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) draw(3)

// Sandwiched Score-based Neighborhood Matching
// with multiple counterfactuals in both directions
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) draw(3) between

// Greedy Score-based Neighborhood Matching
// usage of caliper recommended, especially if draw > 1
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) draw(3) caliper(0.05) greedy

// Score-based Radius Matching
cap drop _*
ultimatch score if period == 0, treated(treated) report(score age weight gender fitness) caliper(0.01) radius

// Coarsened Exact Matching
cap drop _*
ultimatch if period == 0, treated(treated) report(score age weight gender fitness) exact(agegroup weightgroup gender)

// Copying Single Score-based Neighborhood Matching
// alternative method to Coarsened Exact based on pseudo score (allows all score-based options like single, copy, greedy)
// caliper of 0.5 prevents cell transgression
cap drop _*
ultimatch coarsecell if period == 0, treated(treated) report(score age weight gender fitness) caliper(0.5) draw(1) single copy
drop _copy

// Mahalanobis Distance-based Percentile Rank Neighborhood Matching
cap drop _*
ultimatch age weight gender if period == 0, treated(treated) report(score age weight gender fitness) rank mahalanobis

// Euclidean Distance-based Percentile Rank Neighborhood Matching
cap drop _*
ultimatch age weight gender if period == 0, treated(treated) report(score age weight gender fitness) rank euclid

// Euclidean Distance-based Percentile Rank Radius Matching
cap drop _*
ultimatch age weight gender if period == 0, treated(treated) report(score age weight gender fitness) caliper(0.05) rank euclid radius

// Mahalanobis Distance-based Neighborhood Matching
cap drop _*
ultimatch age weight gender if period == 0, treated(treated) report(score age weight gender fitness)

// Supported Mahalanobis Distance-based Neighborhood Matching incl. score (reporting unmatched for reference)
cap drop _*
ultimatch score age weight gender if period == 0, treated(treated) report(score age weight gender fitness) support unmatched

// difference in differences without matching
reg weight treated period effect age gender

// difference in differences with matching
egen m = max(_match), by(id)  // extending the match into the treatment period
egen w = max(_weight), by(id) // extending the weight into the treatment period
reg weight treated period effect age gender [pweight=w] if m != .

